/* ************************************************************************** */
/*                                       _____           _                    */
/*  quit_popup.c                        |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : TuQuoque                  | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/08/11 11:56:08                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#include <keyboard.h>
#include <display.h>
#include <main.h>

/**
 *	qbutton_t:
 *	The buttons enumeration for the quit popup.
 */

typedef enum {
	qbutton_yes,
	qbutton_no,
	max_qbuttons
} qbutton_t;

/**
 *	quit_popup_sel:
 *	Reverses the selected option.
 *
 *	@arg	option			the option code (as in qbutton_t)
 */

static void quit_popup_sel(int option)
{
	if (option == qbutton_yes)
		dreverse_area(DISPLAY_WIDTH / 2 - 32, DISPLAY_HEIGHT / 2 + 3,
			DISPLAY_WIDTH / 2 - 15, DISPLAY_HEIGHT / 2 + 11);
	else
		dreverse_area(DISPLAY_WIDTH / 2 + 14, DISPLAY_HEIGHT / 2 + 3,
			DISPLAY_WIDTH / 2 + 32, DISPLAY_HEIGHT / 2 + 11);
}

/**
 *	quit_popup:
 *	Checks if the user really wants to quit.
 *
 *	I know, again, a whole lot of experimental values.
 *	But you know, as I've got my calc beside ready to test,
 *	well, it's pretty quick to make this functionnal and not so ugly.
 *
 *	@return				the confirmation
 */

int quit_popup(void)
{
	int option = qbutton_no;

	/* initial display */
	/* - no clear, we want to keep the last frame in the background - */
	/* - display big rectangle */
	dclear_area(DISPLAY_WIDTH / 2 - 40, DISPLAY_HEIGHT / 2 - 20,
				DISPLAY_WIDTH / 2 + 41, DISPLAY_HEIGHT / 2 + 21);
	dreverse_area(DISPLAY_WIDTH / 2 - 40, DISPLAY_HEIGHT / 2 - 20,
				DISPLAY_WIDTH / 2 + 41, DISPLAY_HEIGHT / 2 + 21);
	dreverse_area(DISPLAY_WIDTH / 2 - 38, DISPLAY_HEIGHT / 2 - 18,
				DISPLAY_WIDTH / 2 + 39, DISPLAY_HEIGHT / 2 + 19);
	/* - displays the texts - */
	dtext(DISPLAY_WIDTH / 2 - 25, DISPLAY_HEIGHT / 2 - 10, "Quitter ?");
	dtext(DISPLAY_WIDTH / 2 - 31, DISPLAY_HEIGHT / 2 + 4, "Oui");
	dtext(DISPLAY_WIDTH / 2 + 15, DISPLAY_HEIGHT / 2 + 4, "Non");
	/* - highlight selected option - */
	quit_popup_sel(option);
	/* - update - */
	dupdate();

	/* game loop */
	int nobreak = 1;
	while (nobreak) {
		int key = getkey_opt(0, 0);
		switch (key) {
		/* player wants to change selected option */
		case KEY_LEFT:
		case KEY_RIGHT:
			quit_popup_sel(option);
			option += max_qbuttons + (key == KEY_LEFT) ? -1 : 1;
			option %= max_qbuttons;
			quit_popup_sel(option);
			dupdate();
			break;

		/* player wants to quit or select option */
		case KEY_EXIT:
			option = qbutton_yes;
		case KEY_EXE:
			nobreak = 0;
			break;
		}
	}

	/* then return if confirmation has been made */
	return (option == qbutton_yes);
}
