/* ************************************************************************** */
/*                                       _____           _                    */
/*  select_level.c                      |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : TuQuoque                  | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/08/10 16:31:31                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#include <display.h>
#include <keyboard.h>
#include <tales.h>
#include <main.h>


/**
 *	dreverse_area_fix:
 *	Quickfix while we wait for the real function to be corrected.
 *
 *	Real function does not support area completely off the 128x64 zone.
 *	Lephenixnoir is aware of it and will correct this as soon as possible.
 *	In order to continue working, I made this. It's a FIXME.
 *
 *	@arg	x		the abscissa of the first point
 *	@arg	y		the ordinate of the first point
 *	@arg	a		the abscissa of the second point
 *	@arg	b		the ordinate of the second point.
 */

#define min(A, B) ((A) < (B) ? (A) : (B))
#define max(A, B) ((A) > (B) ? (A) : (B))
static void dreverse_area_fix(int x, int y, int a, int b)
{
	if (min(x, a) < DISPLAY_WIDTH && max(x, a) >= 0
	 && min(y, b) < DISPLAY_HEIGHT && max(y, b) >= 0)
		dreverse_area(x, y, a, b);
}
#define dreverse_area(X, Y, A, B) dreverse_area_fix(X, Y, A, B)

/**
 *	map:
 *	Globals containing the map and map ai pointers, useful for when game is
 *	called.
 */

map_t *map = NULL;
map_ai_t *map_ai = NULL;

/**
 *	dthumb:
 *	Draws a level thumbnail.
 *
 *	The thumbnail is 32x32px long. 1 pixel corresponds to 1 tile.
 *	It is directly calculated from the map's data.
 *
 *	@arg	x			the abscissa of the thumbnail
 *	@arg	y			the ordinate of the thumbnail
 *	@arg	map			the map pointer
 */

static void dthumb(int x, int y, map_t *map)
{
	/* make rectangle */
	dreverse_area(x, y, x + 31, y + 31);
	dreverse_area(x + 1, y + 1, x + 30, y + 30);

	/* draw rectangles */
	for (int ry = -14; ry < 14; ry++) { /* "rectangle" y */
		for (int rx = -14; rx < 14; rx++) { /* "rectangle" x */
			/* check tile */
			int ly = ry + map->spawn_y, lx = rx + map->spawn_x; /* "local" */
			int tile = (lx >= 0 && lx < map->w && ly >= 0 && ly < map->h)
				? map->b[ly * map->w + lx] : 1;
			/* draw mini-rectangle if it is the case */
			if (tile) {
				/* "origin" x and y */
				int ox = x + 2 + (rx + 14), oy = y + 2 + (ry + 14);
				dreverse_area(ox, oy, ox, oy);
			}
		}
	}
}

/**
 *	dall:
 *	Draw all that needs to be drawn in selection menu.
 *
 *	Basically, I just thought it would be more tidy in a separate function.
 *	Displays thumbnails and arrows when it needs to.
 *
 *	@arg	index		index of the currently selected map
 */

static void dall(int index)
{
	/* display text */
	dtext(DISPLAY_WIDTH / 2 - 60, 0, "Choisissez le niveau");
	/* left thumb */
	if (index - 1 >= 0) {
		dthumb(-16, 17, maplist[index - 1].map);
		dimage_part(36, 28, &img_arrows, 16, 0, 8, 8); /* left arrow */
	}
	/* center thumb */
	dthumb(49, 17, maplist[index].map);
	dimage_part(61, 48, &img_arrows, 0, 0, 8, 8); /* bottom arrow */
	/* right thumb */
	if (index + 1 <= maplist_size - 1) {
		dthumb(113, 17, maplist[index + 1].map);
		dimage_part(86, 28, &img_arrows, 8, 0, 8, 8); /* right arrow */
	}
}

/**
 *	select_level:
 *	Level selection menu.
 *
 *	Here, the player can choose the level in a list (with thumbnails) in order
 *	to play it. Nothing exceptional, just a nice menu.
 *
 *	@arg	last		last menu
 */

void *select_level(void *last)
{
	(void)last;
	/* initial */
	static int index = 0; /* currently selected level */
	void *menu = NULL;
	mapref_t *mapref;

	/* game loop */
	int nobreak = 1;
	while (nobreak) {
		/* draw all */
		dclear();
		dall(index);
		dupdate();

		/* event gestion */
		int key = getkey_opt(0, 0);
		switch (key) {
		/* - change currently selected level - */
		case KEY_LEFT:
			if (index > 0) index--;
			break;
		case KEY_RIGHT:
			if (index < maplist_size - 1) index++;
			break;

		/* - exit - */
		case KEY_EXIT:
			menu = &main_menu;
			nobreak = 0;
			break;

		/* - go to level - */
		case KEY_EXE:
			/* getting map data */
			mapref = &maplist[index];
			map = mapref->map;
			map_ai = mapref->ai;
			/* set menu and go to it */
			menu = &game;
			nobreak = 0;
			break;
		}
	}

	return (menu);
}
