/* ************************************************************************** */
/*                                       _____           _                    */
/*  sleep.c                             |_   _|__  _   _| |__   ___ _   _     */
/*  | Project : TuQuoque                  | |/ _ \| | | | '_ \ / _ \ | | |    */
/*                                        | | (_) | |_| | | | |  __/ |_| |    */
/*  By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr */
/*  Last updated: 2016/08/09 21:43:57                               |___/     */
/*                                                                            */
/* ************************************************************************** */

#include <timer.h>

/**
 *	sleep_not_done_yet:
 *	Simple global to check if sleep is done.
 */

static volatile int sleep_not_done_yet;

/**
 *	sleep_callback:
 *	Callback for sleep.
 */

static void sleep_callback(void)
{
	sleep_not_done_yet = 0;
}

/**
 *	sleep_some_time:
 *	Sleeps some time.
 *
 *	I don't really have correspondances between ticks and milliseconds.
 *	I check experimentally if the value is correct.
 *
 *	@arg	value		an experimental ticks value
 */

void sleep_some_time(int value)
{
	sleep_not_done_yet = 1;

	/* start timer */
	timer_start(TIMER_USER, value, TIMER_Po_256, &sleep_callback, 0);

	/* loop until the constant is set by the callback */
	do {
		__asm__("sleep\n\t");
	} while (sleep_not_done_yet);

	/* stop timer and we're done */
	timer_stop(TIMER_USER);
}
