#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#******************************************************************************#
#                                        _____           _                     #
#   make-level-object.py                |_   _|__  _   _| |__   ___ _   _      #
#   | Project : TuQuoque                  | |/ _ \| | | | '_ \ / _ \ | | |     #
#                                         | | (_) | |_| | | | |  __/ |_| |     #
#   By: thomas <thomas@touhey.fr>         |_|\___/ \__,_|_| |_|\___|\__, |.fr  #
#   Last updated: 2016/08/08 12:13:40                               |___/      #
#                                                                              #
#******************************************************************************#
# Imports
import os
import sys
import argparse
import subprocess
import tempfile
from tqmap import m

# Argument parser
# - initialize parser
parser = argparse.ArgumentParser(description='Generate a binary map for Tu Quoque')
# - parameter
# --- define custom type converter
def input_file(val):
	if val == '-':
		return os.fdopen(sys.stdin.fileno(), 'r')
	else:
		try:
			return open(val, 'r')
		except IOError:
			raise argparse.ArgumentError(infile, 'could not open input file')
# --- then add argument
parser.add_argument('infile', type=input_file, help='map path')
# - options
# -- prefix
parser.add_argument('-n', dest='prefix',
	type=str, help='symbol prefix', default='map_default')
# -- output
# --- define custom type converter
def output_file(val):
	try:
		return open(val, 'wb')
	except IOError:
		raise argparse.ArgumentError('could not open output file')
	
# --- then add argument
parser.add_argument('-o', dest='outfile',
	type=output_file, help='output file', default='map.o')
# -- another output
parser.add_argument('-eo', dest='eoutfile',
	type=output_file, help='enemy spawns output file', default='map_enemies.o')
# - parse
args = parser.parse_args()

# - Load from file
m.load(args.infile)
args.infile.close()

# - Process
if not m.process():
	exit(1)

# - Generate bytestreams
ms = bytes(m)
mais = bytes(m.enemies)

# - Copy to temporary files
# -- Main map
temp_fd, temp_path = tempfile.mkstemp()
temp = open(temp_path, 'w+b')
temp.write(ms)
temp.flush()
temp.close()
# -- Enemies spawn info
temp2_fd, temp2_path = tempfile.mkstemp()
temp2 = open(temp2_path, 'w+b')
temp2.write(mais)
temp2.flush()
temp2.close()

# - Run subprocesses
section1 = '.rodata.levels,contents,alloc,load,readonly,data'
section2 = '.rodata.levels.enemies,contents,alloc,load,readonly,data'
oldprefix1 = '_binary_{}'.format(temp_path.replace('/', '_'))
oldprefix2 = '_binary_{}'.format(temp2_path.replace('/', '_'))
newprefix1 = '_{}'.format(args.prefix)
newprefix2 = '_{}_enemies'.format(args.prefix)
# -- Main map
subprocess.call(('sh3eb-elf-objcopy',
	'--input', 'binary',
	'--output', 'elf32-sh',
	'--binary-architecture', 'sh3',
	'--file-alignment', '4',
	'--rename-section', '.data={}'.format(section1),
	'--strip-dwo',
	'--redefine-sym', '{}_start={}_start'.format(oldprefix1, newprefix1),
	'--redefine-sym', '{}_end={}_end'.format(oldprefix1, newprefix1),
	'--redefine-sym', '{}_size={}_size'.format(oldprefix1, newprefix1),
	temp_path, args.outfile.name))
# -- Enemies spawn info
subprocess.call(('sh3eb-elf-objcopy',
	'--input', 'binary',
	'--output', 'elf32-sh',
	'--binary-architecture', 'sh3',
	'--file-alignment', '4',
	'--rename-section', '.data={}'.format(section2),
	'--strip-dwo',
	'--redefine-sym', '{}_start={}_start'.format(oldprefix2, newprefix2),
	'--redefine-sym', '{}_end={}_end'.format(oldprefix2, newprefix2),
	'--redefine-sym', '{}_size={}_size'.format(oldprefix2, newprefix2),
	temp2_path, args.eoutfile.name))

# - Delete temporary files
os.remove(temp_path)
os.remove(temp2_path)

# End of file
